Package: ARDL
Type: Package
Title: ARDL, ECM and Bounds-Test for Cointegration
Description: Creates complex autoregressive distributed lag (ARDL) models and 
    constructs the underlying unrestricted and restricted error correction 
    model (ECM) automatically, just by providing the order. It also performs
    the bounds-test for cointegration as described in Pesaran et al. (2001) 
    <doi:10.1002/jae.616> and provides the multipliers and the cointegrating
    equation. The validity and the accuracy of this package have been verified 
    by successfully replicating the results of Pesaran et al. (2001) in 
    Natsiopoulos and Tzeremes (2022) <doi:10.1002/jae.2919>.
Version: 0.2.5
Authors@R: c(
  person("Kleanthis", "Natsiopoulos", 
  email = "klnatsio@gmail.com", role = c("aut", "cre"), 
  comment = c(ORCID = "0000-0003-1180-2984")), 
  person("Nickolaos", "Tzeremes", 
  email = "bus9nt@econ.uth.gr", role = "ths",
  comment = c(ORCID = "0000-0002-6938-3404")),
  person("Daniel", "Finnan", 
  email = "dan@custom-made.org.uk", role = "aut",
  comment = c(ORCID = "0009-0009-7468-5288")))
BugReports: https://github.com/Natsiopoulos/ARDL/issues
License: GPL-3
URL: https://github.com/Natsiopoulos/ARDL
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Suggests: strucchange, tseries, AICcmodavg, sandwich, testthat (>=
        3.0.0)
Imports: aod, dplyr, dynlm, gridExtra, ggplot2, lmtest, msm, stringr,
        zoo, stats
RoxygenNote: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-09 20:19:35 UTC; KNatsiopoulos001
Author: Kleanthis Natsiopoulos [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1180-2984>),
  Nickolaos Tzeremes [ths] (ORCID:
    <https://orcid.org/0000-0002-6938-3404>),
  Daniel Finnan [aut] (ORCID: <https://orcid.org/0009-0009-7468-5288>)
Maintainer: Kleanthis Natsiopoulos <klnatsio@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-10 05:12:54 UTC
Built: R 4.4.3; ; 2026-05-10 06:41:21 UTC; unix
