# Generated by roxygen2: do not edit by hand

S3method(ardl,default)
S3method(ardl,uecm)
S3method(coint_eq,default)
S3method(coint_eq,recm)
S3method(delta_method,ardl)
S3method(delta_method,uecm)
S3method(multipliers,ardl)
S3method(multipliers,uecm)
S3method(predict,ardl)
S3method(uecm,ardl)
S3method(uecm,default)
export(ardl)
export(auto_ardl)
export(bounds_f_test)
export(bounds_t_test)
export(coint_eq)
export(multipliers)
export(plot_delay)
export(plot_lr)
export(recm)
export(to_lm)
export(uecm)
importFrom(dplyr,"%>%")
importFrom(zoo,merge.zoo)
